/***********************************************************************
*   CHPASS.C - Sets passwords for users listed in a file
*
*   MUST be compiled in small model with byte alignment of structures.
*       eg. Microsoft
*           CL /Zp1 /AS OWNER.C
*
*   Chris Owen, Queensland University of Technology, Brisbane, Australia
*   10/1/90
*/

#include <stdio.h>
#include <dos.h>
#include <process.h>
#define CHANGE_OBJECT_PASSWORD 0xE300  /* Novell call - Change Object Password */


int usage (char *msg)
{
    if (*msg && msg)
        printf ("%s\n", msg);
    printf ("CHPASS - Takes a control file of (username,password) pairs and resets passwords\n");
    printf ("         from this.  For Novell NetWare, doesn't use NW386's password encryption\n");
    printf ("         so ensure that you SET ALLOW ENCRYPTED PASSWORDS=ON at the console.\n\n");
    printf ("usage CHPASS <filename>\n");
    printf ("\twhere <filename> is a file of lines containing <username> <password>\n");
    printf ("\tpairs.\n\n");
    printf ("Chris Owen, QUT 10/1/91\n");
    exit (1);
}

int ChangeObjectPassword (char *object, char *oldpassword, char *newpassword)
{
    union REGS   regs;
    struct SREGS sregs;
    struct{
        int  len;
        char fun;
        char type_hi,
             type_lo,
             data[100];
        }req_buff;
    int rep_buff=0;
    int i, offset=0;

    /* ensure everything is in upper case */
    for (i=0; object[i]; i++)
        object[i]      = toupper(object[i]);
    for (i=0; oldpassword[i]; i++)
        oldpassword[i] = toupper(oldpassword[i]);
    for (i=0; newpassword[i]; i++)
        newpassword[i] = toupper(newpassword[i]);

    regs.x.ax = CHANGE_OBJECT_PASSWORD;             /* set function */

    req_buff.len = 6 + strlen (object)              /* set buffer length */
                    + strlen (oldpassword)
                    + strlen (newpassword);
    req_buff.fun = 0x40;                            /* set subfunction */
    req_buff.type_hi = 0;                           /* set object type = USER */
    req_buff.type_lo = 1;

    req_buff.data[0] = strlen (object);             /* object name (length first) */
    strcpy (&req_buff.data[1], object);
    offset += req_buff.data[offset] + 1;
    
    req_buff.data[offset] = strlen (oldpassword);   /* old password (length first) */
    strcpy (&req_buff.data[offset+1], oldpassword);
    offset += req_buff.data[offset] + 1;

    req_buff.data[offset] = strlen (newpassword);   /* new password (length first) */
    strcpy (&req_buff.data[offset+1], newpassword);

    regs.x.si = (int) &req_buff;        /* note this type cast is NOT PORTABLE */
    regs.x.di = (int) &rep_buff;        /* between memory models */
    intdos (&regs, &regs);                          /* call NetWare API */
    return (regs.h.al);                             /* error code in AL */
}

int main (int argc, char *argv[])
{
    FILE *list;
    char name[100], password[100];
    unsigned char errcode;

    if (argc != 2)
        usage ("arg count!");
    if ((list = fopen (argv[1], "r")) == NULL){
        perror (argv[1]);
        usage (NULL);
        }

    /* do each (name, password) pair */
    while (fscanf (list, "%s %s", name, password) != EOF){
        printf ("%s: ", name);
        errcode = ChangeObjectPassword (name, "", password);
        if (!errcode)
            printf ("OK - password set\n");
        else{
            printf ("password not set ... ");
            switch (errcode){
                case 0xD6:
                    printf("Unencrypted password calls not allowed\n");
                    printf("SET ALLOW UNENCRYPTED PASSWORDS=ON on the file server console and try again\n");
                    break;
                case 0xD7:
                    printf ("Password is a duplicate of a previously used password\n");
                    break;
                case 0xD8:
                    printf ("Password is too short\n");
                    break;
                case 0x96:
                    printf("Server Out Of Memory\n");
                    break;
                case 0xF0:
                    printf("Wildcard Not Allowed\n");
                    break;
                case 0xFB:
                    printf("No Such Property\n");
                    break;
                case 0xFC:
                    printf("No Such User\n");
                    break;
                case 0xFE:
                    printf("Server Bindery Locked\n");
                    break;
                case 0xFF:
                    printf("No Such User, or old password (NULL) bad\n");
                    break;
                default:
                    printf("unknown error %X\n", errcode);
                }
            }
        }
    fclose (list);
}
